#!/usr/bin/env python
# -*-coding:utf-8-*-
# 测验正常的rpc服务器

import unittest
from xnrpc.server import RpcInterface, ServerManager
from xnrpc.client import open_client
from xnrpc.common.log import get_log
_log = get_log(__name__)


class OperationRpcInterface(RpcInterface):

    def ls_file(self):
        _log.info('Operation ls file')
        return None

    def check_ip(self):
        _log.info('Operation check ip')
        return None


class RpcTest(unittest.TestCase):
    def test_server_manager(self):
        # 启动RPC服务器
        rpc_interface = OperationRpcInterface
        url = 'tcp://127.0.0.1:7772'
        server = ServerManager(url, rpc_interface, name='Test Server')
        server.start()

        # 建立一个客户端连接
        with open_client(url) as client:
            client.ls_file()
            client.check_ip()

        # 停止服务器
        server.stop()


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# -*-coding:utf-8-*-

import datetime
import gevent.greenlet


def now():
    """  A non-tz-aware now  """
    return datetime.datetime.now()

def now_str():
    return now().strftime('%Y-%m-%d %H:%M:%S')

def format_time(timeinfo):
    return datetime.datetime.fromtimestamp(timeinfo).strftime('%Y-%m-%d %H:%M:%S')


class Ticker(gevent.greenlet.Greenlet):
    def __init__(self, period, callback, *args, **kwargs):
        super(Ticker, self).__init__(*args, **kwargs)
        self._period = period
        self._callback = callback
        self._complete = gevent.event.Event()

    def stop(self):
        self._complete.set()

    def _run(self):
        while not self._complete.is_set():
            self._callback()
            self._complete.wait(self._period)




#!/usr/bin/env python
# -*-coding:utf-8-*-

import gevent.event
import gevent.greenlet
from xnrpc.common.log import get_log
_log = get_log(__name__)

WAIT_INTERVAL = 5


class ServerThread(gevent.greenlet.Greenlet):
    """gevent服务器协程"""
    def __init__(self):
        super(ServerThread, self).__init__()
        self._complete = gevent.event.Event()

    def _run(self):
        _log.info("Start %s" % self.__class__.__name__)
        while not self._complete.is_set():
            try:
                _log.debug("Enter...")
                self.work()
            except:
                _log.error("Work exception...", exc_info=1)
            finally:
                self._complete.wait(WAIT_INTERVAL)
        _log.info("Completed %s" % self.__class__.__name__)

    def work(self):
        """覆盖这个方法"""
        pass

    def stop(self):
        _log.info("%s stopping", self.__class__.__name__)
        self._complete.set()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
Filter(logname)，只允许来自logname或其子日志的消息通过app.net是app的子日志
消息传播propagate和分层记录器：消息会传播给父记录器，log.propagate属性获取是否传播标志
在Python中正确的记录Log方式应该是这样的：
    logging.exception(ex)
    logging.error(ex, exc_info=1) # 指名输出栈踪迹, logging.exception的内部也是包了一层此做法
    logging.critical(ex, exc_info=1) # 更加严重的错误级别
"""
import logging
import logging.handlers as handlers
import logging.config as config

# 时间 | 日志级别 | 文件名 | 行号 | 进程ID | Logger名 | 消息体
_fmt = "%(asctime)-15s [%(levelname)s] %(filename)s line=%(lineno)d" \
      " pid=%(process)d %(name)s ===> %(message)s"
_datefmt = "%Y-%m-%d %H:%M:%S"
_formatter = logging.Formatter(_fmt, _datefmt)
_log_dict = {
    'default': '/var/log/default.log',
    'winstore.db': '/var/log/db.log',
}


def _get_key(module_name):
    """根据模块名获取日志key"""
    mlist = module_name.split('.')
    mlen = len(mlist)
    while mlen > 1:
        cutkey = '.'.join(mlist[:mlen])
        if cutkey in _log_dict:
            return cutkey
        mlen -= 1
    return "default"


def get_log(name, level=logging.DEBUG):
    logfile = _log_dict[_get_key(name)]
    _log = logging.getLogger(name)
    _log.propagate = False  # 关闭传播属性
    _log.setLevel(level)
    # 7天轮换一次日志文件
    _handler = logging.handlers.TimedRotatingFileHandler(logfile, when='D', interval=7)
    _handler.setFormatter(_formatter)
    _log.addHandler(_handler)
    return _log

if __name__ == '__main__':
    """main test"""
    _log = get_log(__name__)
    _log.info('this is main.')
    pass



#!/usr/bin/env python
# -*-coding:utf-8-*-

import time
import gevent
import zerorpc
from xnrpc.common.log import get_log
_log = get_log(__name__)


class ServerManager(object):
    def __init__(self, url, rpc_interface, heartbeat=None, name=None):
        self.rpc_server_last_service_time = None
        self.rpc_server = zerorpc.Server(rpc_interface(self), heartbeat=heartbeat)
        self.rpc_server.bind(url)
        self.rpc_gthread = None
        self.name = name

    def start(self):
        _log.info("{} starting".format(self.name))
        self.rpc_gthread = gevent.spawn(self.rpc_server.run)
        _log.info("{} rpc server started".format(self.name))

    def stop(self):
        _log.info("{} stopping".format(self.name))
        self.rpc_server.stop()
        _log.info("{} rpc server stopped".format(self.name))

    def join(self):
        self.rpc_gthread.join()


class RpcInterface(object):
    def __init__(self, manager=None):
        self._manager = manager
        if self._manager and not self._manager.rpc_server_last_service_time:
            self._manager.rpc_server_last_service_time = time.time()
        # 100 ms is too slow
        self.SLOW_THRESHOLD = 0.1

    def __getattribute__(self, item):
        """
        Wrap functions with logging
        """
        if item.startswith('_'):
            return object.__getattribute__(self, item)
        else:
            attr = object.__getattribute__(self, item)
            if callable(attr):
                def wrap(*args, **kwargs):
                    _log.debug("RpcInterface >> %s", item)
                    try:
                        begin_time = time.time()
                        if self._manager:
                            self._manager.rpc_server_last_service_time = begin_time
                        rc = attr(*args, **kwargs)
                        end_time = time.time()
                        elapse_time = end_time - begin_time
                        if elapse_time > self.SLOW_THRESHOLD:
                            _log.warn("Slow proccess '%s', %sms", item, elapse_time * 1000)
                        else:
                            _log.debug("RpcInterface << %s %sms", item, elapse_time * 1000.0)
                    except:
                        _log.exception("RpcInterface exception, %s" % item)
                        raise
                    return rc

                return wrap
            else:
                return attr




#!/usr/bin/env python
# -*-coding:utf-8-*-
# 测验gevent协程

import unittest
from xnrpc.common.thread import ServerThread
from xnrpc.common.log import get_log
_log = get_log(__name__)


class OperationThread(ServerThread):

    def __init__(self, counter = 1):
        super(OperationThread, self).__init__()
        self.counter = counter

    def work(self):
        _log.info('OperationThread, counter={}'.format(self.counter))
        _log.info('OperationThread, counter={}'.format(self.counter + 1))
        _log.info('OperationThread, counter={}'.format(self.counter + 2))


class ThreadTest(unittest.TestCase):
    def test_operation_thread(self):
        t = OperationThread(counter=3)
        t.start()

        import time
        time.sleep(3)

        # now stop thread
        t.stop()




#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import shlex
import gevent
from gevent.subprocess import Popen, PIPE
from xnrpc.common.log import get_log
_log = get_log(__name__)


def proc_cmd(cmd, **kwargs):
    """
    :param cmd:
    数组形式: ['ceph', 'osd', 'dump', '|', 'grep', 'poolname']
    字符串形式: 'ceph osd dump | grep "poolname"'
    kwargs传递其他额外参数，比如timeout=60，sudo=True等
    """
    sudo = kwargs.get('sudo', True)
    timeout = kwargs.get('timeout', 20)
    # 字符串命令转换成数组
    if isinstance(cmd, str):
        cmd = shlex.split(cmd)
    _log.info('proc_cmd command: {}'.format(cmd))
    # 为了处理好管道的情况，程序限定 '|' 分割写法
    all_cmd_list = list(_split_list(cmd, '|'))
    len_arg = len(all_cmd_list)
    if len_arg == 1:
        cmd_args = all_cmd_list[0]
        if sudo:
            cmd_args.insert(0, 'sudo')
        proc = Popen(cmd_args, close_fds=True, stdout=PIPE, stderr=PIPE)
        try:
            with gevent.Timeout(timeout, False):
                stdout, stderr = proc.communicate()
            if proc.returncode is None:
                _log.error("gevent.Timeout.cmd={},timeout={}".format(cmd, timeout))
                if proc:
                    kill_p = Popen(['sudo', 'kill', '--', str(proc.pid)], close_fds=True)
                    kill_p.wait()
                return 2, '', 'timeout error'
            return proc.returncode, stdout, stderr
        except Exception:
            if proc:
                # proc.kill()
                kill_p = Popen(['sudo', 'kill', '--', str(proc.pid)], close_fds=True)
                kill_p.wait()
            _log.error("proc_cmd unknownerror: cmd={}".format(cmd), exc_info=1)
            return 3, '', 'proc_cmd unknown error'
    else:
        if sudo:
            all_cmd_list[0].insert(0, 'sudo')
        p1 = Popen(all_cmd_list[0], stdout=PIPE)
        for cmd_middle in all_cmd_list[1:-1]:
            if sudo:
                cmd_middle.insert(0, 'sudo')
            p2 = Popen(cmd_middle, stdin=p1.stdout, stdout=PIPE, stderr=PIPE)
            p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
            p1 = p2
        if sudo:
            all_cmd_list[-1].insert(0, 'sudo')
        p2 = Popen(all_cmd_list[-1], stdin=p1.stdout, stdout=PIPE, stderr=PIPE)
        p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
        try:
            with gevent.Timeout(timeout, False):
                stdout, stderr = p2.communicate()
            if p2.returncode is None:
                if p2:
                    # p2.kill()
                    kill_p = Popen(['sudo', 'kill', '--', str(p2.pid)], close_fds=True)
                    kill_p.wait()
                _log.error("gevent.Timeout.cmd={},timeout={}".format(cmd, timeout))
                return 2, '', 'timeout error'
        except Exception as e:
            if p2:
                # p2.kill()
                kill_p = Popen(['sudo', 'kill', '--', str(p2.pid)], close_fds=True)
                kill_p.wait()
            _log.error("proc_cmd unknownerror: cmd={}".format(cmd), exc_info=1)
            return 3, '', 'proc_cmd unknown error'
        return p2.returncode, stdout, stderr


def _split_list(lst, symbol):
    item = []
    for i, e in enumerate(lst):
        if e == symbol:
            yield item
            item = []
        elif i == len(lst) - 1:
            item.append(str(e))
            yield item
        else:
            item.append(str(e))




#!/usr/bin/env python
# -*-coding:utf-8-*-
# 复杂一点的测试
# manager中有rpc server和多个thread
# rpc interface会调用manager中thread的方法

import unittest
from xnrpc.server import RpcInterface, ServerManager
from xnrpc.client import open_client
from xnrpc.common.thread import ServerThread
from xnrpc.common.log import get_log
_log = get_log(__name__)


class OperationThread(ServerThread):

    def __init__(self, counter = 1):
        super(OperationThread, self).__init__()
        self.counter = counter

    def work(self):
        _log.info('OperationThread, counter={}'.format(self.counter))
        _log.info('OperationThread, counter={}'.format(self.counter + 1))
        _log.info('OperationThread, counter={}'.format(self.counter + 2))


class OperationRpcInterface(RpcInterface):

    def ls_file(self):
        """两种调用方法"""
        _log.info('Operation ls file, now I call thread method')
        self._manager.opthread.work()
        return None

    def check_ip(self):
        """管道命令"""
        _log.info('Operation check ip, now I do nothing')
        return None


class OperationServerManager(ServerManager):
    def __init__(self, url, rpc_interface):
        super(OperationServerManager, self).__init__(
            url, rpc_interface, name='OperationServerManager')
        self.opthread = OperationThread(counter=100)

    def start(self):
        super(OperationServerManager, self).start()
        self.opthread.start()

    def stop(self):
        super(OperationServerManager, self).stop()
        self.opthread.stop()

    def join(self):
        super(OperationServerManager, self).join()
        self.opthread.join()


class RpcTest(unittest.TestCase):
    def test_server_manager(self):
        # 启动RPC服务器
        rpc_interface = OperationRpcInterface
        url = 'tcp://127.0.0.1:7779'
        server = OperationServerManager(url, rpc_interface)
        server.start()

        # 建立一个客户端连接
        with open_client(url) as client:
            client.ls_file()
            client.check_ip()

        # 停止服务器
        server.stop()


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='xnrpc',
    version='1.2.2',
    packages = find_packages(),
    # Project uses , so ensure
    install_requires=[
        "gevent>=1.1.2",
        "zerorpc>=0.6.0",
    ],
    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
        # include any *.msg files found in the 'test' package, too:
        'test': ['*.msg'],
    },
    description='simple rpc based on zerorpc and gevent',
    long_description=open("README.rst").read(),
    url='https://github.com/yidao620c/xnrpc',
    author='Xiong Neng',
    author_email='yidao620@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords=['xnrpc', 'gevent', 'zerorpc', 'subprocess'],
)




#!/usr/bin/env python
#-*-coding:utf-8-*-
"""
The Resource class is a base class for all resource type.
"""
import time
import gevent
import zerorpc
from collections import defaultdict
from contextlib import contextmanager
from xnrpc.common.log import get_log

_log = get_log(__name__)


@contextmanager
def open_client(url, timeout=10, heartbeat=None):
    client = RpcClient(url, timeout=timeout, heartbeat=heartbeat)
    try:
        yield client
    except:
        _log.error('rpc client error.', exc_info=1)
    finally:
        client.close()


class RpcClient(zerorpc.Client):

    def __init__(self, url, slow_threshold=0.2,
                 max_perf_count=10, profile=False, timeout=10, heartbeat=None):
        super(RpcClient, self).__init__(timeout=timeout, heartbeat=heartbeat)
        self.URL = url
        self.slow_threshold = slow_threshold
        self.max_perf_count = max_perf_count
        self.profile = profile
        self.method_times = defaultdict(list)
        _log.debug("Conneting %s", self.URL)
        self.connect(self.URL)

    def _process_response(self, request_event, bufchan, timeout):
        begin_time = time.time()
        result = super(RpcClient, self)._process_response(request_event, bufchan, timeout)
        end_time = time.time()
        elapse_time = end_time - begin_time
        _log.debug("RPC elapse time '%s' %sms", request_event.name, elapse_time * 1000)
        if elapse_time > self.slow_threshold:
            _log.warn("Slow RPC '%s' %sms", request_event.name, elapse_time * 1000)

        if self.profile:
            self.method_times[request_event.name].append(elapse_time)
            self._report()
        return result

    def _report(self):
        for method_name, times in self.method_times.items():
            if len(times) < self.max_perf_count:
                continue
            _log.debug("RPC timing for '%s': %s/%s/%s avg/max/min(ms) times: %s",
                      method_name,
                      sum(times) * 1000.0 / len(times),
                      max(times) * 1000.0,
                      min(times) * 1000.0,
                      len(times))
            if len(times) >= self.max_perf_count:
                self.method_times[method_name] = []


class TimeoutRpcClient(object):
    def __init__(self, name, rpc_new_fn, default_time_out=10, elaspe_time_step=5):
        self._name = name
        self._new_instance_fn = rpc_new_fn
        self._event_wait = gevent.event.Event()
        self._default_time_out = default_time_out
        if self._default_time_out <= 0:
            self._default_time_out = 60
        self._elaspe_time_step = elaspe_time_step
        if self._elaspe_time_step < 0:
            self._elaspe_time_step = 5
        self._instance = None
        self._timeout = 0

    def get_instance(self):
        if not self._instance:
            _log.info("New rpc %s", self._name)
            self._instance = self._new_instance_fn()
            gevent.spawn(self._close_instance)
        self._timeout = self._default_time_out
        return self._instance

    def _close_instance(self):
        while True:
            self._event_wait.wait(self._elaspe_time_step)
            self._timeout -= self._elaspe_time_step
            if self._timeout <= 0:
                break
        _log.info("Close rpc %s", self._name)
        if self._instance:
            self._instance.close()
        self._instance = None
        self._timeout = 0